#include "PackageTableActionContextMenu.h"
#include "PackageTableView.h"
#include "Share.h"

PackageTable::ActionContextMenu::ActionContextMenu(Model*  model,
                                                   View*   view)
   : QMenu(),
     noneIcon(),
     updateIcon(Share::absolute("icons/update.png")),
     installIcon(Share::absolute("icons/install.png")),
     removeIcon(Share::absolute("icons/remove.png")),
     model(model),
     view(view)
{
   updateAction  = new QAction(tr("&Update"), this);
   installAction = new QAction(tr("&Install"), this);
   removeAction  = new QAction(tr("&Remove"), this);
   noneAction    = new QAction(tr("&No Action"), this);

   updateAction->setIcon(updateIcon);
   installAction->setIcon(installIcon);
   removeAction->setIcon(removeIcon);
   noneAction->setIcon(noneIcon);

   addAction(updateAction);
   addAction(installAction);
   addAction(removeAction);
   addAction(noneAction);

   connect(updateAction, SIGNAL(triggered()), 
           this, SLOT(handleUpdateAction()));
   connect(installAction, SIGNAL(triggered()), 
           this, SLOT(handleInstallAction()));
   connect(removeAction, SIGNAL(triggered()), 
           this, SLOT(handleRemoveAction()));
   connect(noneAction, SIGNAL(triggered()), 
           this, SLOT(handleNoneAction()));
}

void  PackageTable::ActionContextMenu::exec(QModelIndex    ix,
                                            const QPoint&  pos)
{
   this->ix = ix;

   bool  installEnable = false;
   bool  updateEnable  = false;
   bool  removeEnable  = false;

#if QT_VERSION >= 0x040200
   if (view->selectionModel()->hasSelection()) {
#else
   if (!view->selectionModel()->selection().isEmpty()) {
#endif
      // Enable menu items if any of the selected rows can use them.
      const QModelIndexList&           sel = view->selectedRows();
      QModelIndexList::const_iterator  it;

      for (it = sel.begin(); it != sel.end(); ++it) {
         Package::State  state = model->getState(it->row());

         switch (state) {
         case Package::OUT_OF_DATE: 
            updateEnable  = true;
            removeEnable  = true;
            break;
         case Package::NOT_INSTALLED:
            installEnable = true;
            break;
         case Package::INSTALLED:
            removeEnable  = true;
            break;
         }
      }
   } else {
      Package::State  state = model->getState(ix.row());

      switch (state) {
      case Package::OUT_OF_DATE: 
         updateEnable  = true;
         removeEnable  = true;
         break;
      case Package::NOT_INSTALLED:
         installEnable = true;
         break;
      case Package::INSTALLED:
         removeEnable  = true;
         break;
      }
   }

   installAction->setEnabled(installEnable);
   updateAction->setEnabled(updateEnable);
   removeAction->setEnabled(removeEnable);

   QMenu::exec(pos);
}

void  PackageTable::ActionContextMenu::handleUpdateAction()
{
   handleAction(Package::UPDATE);
}

void  PackageTable::ActionContextMenu::handleInstallAction()
{
   handleAction(Package::INSTALL);
}

void  PackageTable::ActionContextMenu::handleRemoveAction()
{
   handleAction(Package::REMOVE);
}

void  PackageTable::ActionContextMenu::handleNoneAction()
{
   handleAction(Package::NONE);
}

void  PackageTable::ActionContextMenu::handleAction(Package::Action  act)
{
#if QT_VERSION >= 0x040200
   if (view->selectionModel()->hasSelection()) {
#else
   if (!view->selectionModel()->selection().isEmpty()) {
#endif
      const QModelIndexList&           sel = view->selectedRows();
      QModelIndexList::const_iterator  it;

      for (it = sel.begin(); it != sel.end(); ++it) {
         const QModelIndex&  ix    = *it;
         Package::State      state = model->getState(ix.row());

         switch (state) {
         case Package::OUT_OF_DATE: 
            if (act == Package::UPDATE
                || act == Package::REMOVE
                || act == Package::NONE) {
               model->setAction(ix, act);
            }
            break;
         case Package::NOT_INSTALLED:
            if (act == Package::INSTALL || act == Package::NONE) {
               model->setAction(ix, act);
            }
            break;
         case Package::INSTALLED:
            if (act == Package::REMOVE || act == Package::NONE) {
               model->setAction(ix, act);
            }
            break;
         }
      }
   } else {
      // Only the allowable buttons were enabled.  No need to verify them.
      model->setAction(ix, act);
   }
}
